# Script to use a tarball to push to OBS to trigger builds.
# Note that this script is intentionally designed to be generic and usable
# for both public and private OBS instances.

from __future__ import print_function

import click

import sys
import os
import tempfile
import tarfile
import shutil
import pathlib
import glob
import datetime

from obsctl.cli import pass_context


# Check for the existence of osc executible
osc_check = pathlib.Path("/usr/bin/osc").exists()

if osc_check is not True:
    # Exit if osc is not installed
    print("The Open Build Service Commander is not installed!")
    sys.exit(2)
else:
    # Import osc modules
    import osc.conf
    import osc.core


@click.command('tarsrc-obsimport', help='Push code to an OBS instance.')
@click.option('--source-file', '-f', help='Path to source tarball file')
@click.option('--continuous-merge', '-m', help='Enable continuous merge mode',
              is_flag=True)
@click.option('--repo-type', '-T', help='Source repository type',
              type=click.Choice(['git', 'hg', 'svn']), required=True)
@click.option('--repo-owner', '-o', help='Source repository owner',
              required=True)
@click.option('--repo-name', '-n', help='Source repository name')
@click.option('--repo-reltag', '-t', help='Source repository release tag',
              required=True)
@click.option('--repo-revision', '-R',
              help='Source repository revision/commit', required=True)
@click.option('--repo-branch', '-B', help='Source repository branch',
              required=True)
@click.option('--revision-form', '-r', help='Form of version for spec',
              type=click.Choice(['prerel', 'postrel']), default='postrel')
@click.option('--obs-project', '-s', help='Open Build Service project space',
              required=True)
@click.option('--obs-package', '-p', help='Open Build Service project package',
              required=True)
@click.option('--create-obs-pkg-if-none', '-c',
              help='Create package space in OBS if it doesn\'t exist',
              is_flag=True)
@click.option('--insecure', '-i',
              help='Ignore issues with HTTPS (self signed certs, etc.)',
              is_flag=True)
@pass_context
def cli(ctx, source_file, continuous_merge, repo_type, repo_owner, repo_name,
        repo_reltag, repo_revision, repo_branch, revision_form, obs_project,
        obs_package, create_obs_pkg_if_none, insecure):
    # TODO: Clean this up
    args = {
            'src_tarball': source_file,
            'ci_mode': continuous_merge,
            'src_repo_type': repo_type,
            'src_repo_owner': repo_owner,
            'src_repo_name': repo_name,
            'src_repo_tag': repo_reltag,
            'src_repo_rev': repo_revision,
            'src_repo_branch': repo_branch,
            'spec_rev_form': revision_form,
            'obs_server_addr': ctx.obs_server,
            'obs_auth_user': ctx.obs_user,
            'obs_auth_pass': ctx.obs_pass,
            'obs_project': obs_project,
            'obs_package': obs_package,
            'create_obs_pkg_if_none': create_obs_pkg_if_none,
            'insecure': insecure
            }

    # Configure the Open Build Service Commander

    # SSL check setting
    obs_ssl_check = "1"
    if args['insecure'] is True:
            obs_ssl_check = "0"

    # Open Build Service Commander configuration file data
    obs_server_config = """
    [general]
    apiurl = {0}
    do_package_tracking = 1


    [{0}]
    user = {1}
    pass = {2}
    keyring = 0
    sslcertck = {3}
    """.format(args['obs_server_addr'], args['obs_auth_user'],
               args['obs_auth_pass'], obs_ssl_check)

    obs_newpkg_config = """
    <package name="{1}" project="{0}">
      <title>{1} for {0}</title>
      <description>{1} for {0} created by {2}. Code is at {3}/{4}</description>
    </package>
    """.format(args.obs_project, args['obs_package'], args['obs_auth_user'],
               args['src_repo_owner'], args['src_repo_name'])

    # Make a temporary config file to set up for using with osc and use it
    with tempfile.NamedTemporaryFile(suffix=".oscrc") as oscconf:
            oscconf.write(obs_server_config)
            oscconf.flush()
            osc.conf.get_config(override_conffile=oscconf.name)

    # Check to see if the package space exists on OBS
    osc_prj_packagelist = list(osc.core.meta_get_packagelist(
                                            args['obs_server_addr'],
                                            prj=args['obs_project']
                                        ))

    if args['obs_package'] in osc_prj_packagelist:
            print("Package is configured remotely, proceeding to upload...")
    else:
        # If it doesn't exist and we are allowing arbitrary package
        # space creation, create the package space and proceed onward
        if args['create_obs_pkg_if_none'] is True:
            print("Package is not configured remotely, constructing package!")
            osc.core.edit_meta(metatype="pkg", data=obs_newpkg_config,
                               edit=False, apiurl=args['obs_server_addr'],
                               path_args=(args['obs_project'],
                                          args['obs_package']))
            print("Package space created, continuing to upload...")
        else:
            print("Package is not configured remotely, exiting!")
            oscconf.close()
            sys.exit(4)

    # Create directory to check out package sources
    pkg_checkout_dir = tempfile.mkdtemp(suffix="{0}-{1}"
                                        .format(args['obs_project'],
                                                args.obs_package))

    # Check out the package sources
    osc.core.checkout_package(apiurl=args['obs_server_addr'],
                              project=args['obs_project'],
                              package=args['obs_package'],
                              outdir=pkg_checkout_dir)

    # Open OBS repo data
    osc_working_pkg = osc.core.Package(workingdir=pkg_checkout_dir)

    # Purge all tarballs and the _service file
    prjfiles = glob.glob("{0}/*".format(pkg_checkout_dir))

    for pf in prjfiles:
        osc_working_pkg.delete_file(pathlib.PurePath(pf).name)

    # Generate a short hash for hg/git, reuse rev for subversion
    if args['src_repo_type'] == "git" or args['src_repo_type'] == "hg":
        repo_tag_shorthash = args.src_repo_rev[:7]
    else:
        repo_tag_shorthash = args.src_repo_rev

    # Generate the datetime stamp for the build
    pkg_build_datetime = datetime.datetime.now()

    pkg_build_date = "{0}{1}{2}".format(pkg_build_datetime.year,
                                        pkg_build_datetime.month,
                                        pkg_build_datetime.day)
    pkg_build_time = "{0}{1}".format(pkg_build_datetime.hour,
                                     pkg_build_datetime.minute)
    # pkg_build_time = "{0}{1}{2}".format(pkg_build_datetime.hour, pkg_build_datetime.minute, pkg_build_datetime.second)

    # Configure version form
    # 0: Version, 1: VCS, 2: Build date, 3: build time, 4: VCS revision

    if args['spec_rev_form'] == "postrel":
        spec_ver_form = "{0}+{1}{2}{3}.{4}"
    elif args['spec_rev_form'] == "prerel":
        spec_ver_form = "{0}.0.1~{1}{2}{3}.{4}"
    else:
        spec_ver_form = "{0}.{1}{2}{3}.{4}"

    # Rebuild the tarball properly
    orig_tarball = tarfile.open(name=args['src_tarball'], mode="r:gz")
    tarfix_dir = "{0}/tarfix".format(pkg_checkout_dir)
    os.mkdir(tarfix_dir)
    orig_tarball.extractall(path=tarfix_dir)
    orig_tarball.close()

    if args['ci_mode'] is True:
        release_tarball_ver = spec_ver_form.format(args['src_repo_tag'],
                                                   args['src_repo_type'],
                                                   pkg_build_date,
                                                   pkg_build_time,
                                                   repo_tag_shorthash)
    else:
        release_tarball_ver = args['src_repo_tag']

    rebuilt_tarball_prefix = "{0}-{1}".format(args['src_repo_name'],
                                              release_tarball_ver)
    rebuilt_tarball_name = "{0}.tar.gz".format(rebuilt_tarball_prefix)
    shutil.move("{0}/{1}".format(tarfix_dir, args['src_repo_name']),
                "{0}/{1}-{2}".format(tarfix_dir, args['src_repo_name'],
                                     release_tarball_ver))

    shutil.make_archive(base_name="{0}/{1}".format(pkg_checkout_dir,
                                                   rebuilt_tarball_prefix),
                        format="gztar", root_dir=tarfix_dir,
                        base_dir="./{0}-{1}".format(args['src_repo_name'],
                                                    release_tarball_ver))
    shutil.rmtree(tarfix_dir)

    print("{0} has been rebuilt.".format(rebuilt_tarball_name))

    # TODO: This is never used?
    # gen_tarball_dl_path = "{0}/{1}".format(pkg_checkout_dir, rebuilt_tarball_name)

    # Auto-set the version
    if args['ci_mode'] is True:
            obs_version_service_chunk = """
      <service name="set_version">
        <param name="version">{0}</param>
        <param name="file">_service:extract_file:{1}.spec</param>
      </service>""".format(release_tarball_ver, args['obs_package'])
    else:
            obs_version_service_chunk = ""

    # Service file for retrieving the spec from the tarball
    obs_package_service = """
    <services>
      <service name="extract_file">
        <param name="archive">{0}*.tar.gz</param>
        <param name="files">*{0}*/dist/{1}.spec</param>
      </service>{2}
    </services>
    """.format(args['src_repo_name'], args['obs_package'],
               obs_version_service_chunk)

    # Write service file out
    obs_package_service_file = open(pkg_checkout_dir + "/_service", 'w')
    obs_package_service_file.write(obs_package_service)
    obs_package_service_file.close()

    # Send changes to OBS

    # Add files
    try:
            osc_working_pkg.addfile(rebuilt_tarball_name)
    except osc.oscerr.PackageFileConflict:
            print("Already tracked tarball... continuing!")
            pass
    try:
            osc_working_pkg.addfile("_service")
    except osc.oscerr.PackageFileConflict:
            print("Already tracked service file... continuing!")
            pass

    # Commit changes
    osc_working_pkg.commit(msg="Committed {0}/{1}-{2}".format(
                                                args['src_repo_owner'],
                                                args['src_repo_name'],
                                                release_tarball_ver
                                            ))

    # Delete code checkout and temp config files
    shutil.rmtree(pkg_checkout_dir)
    oscconf.close()
